# -*- coding: utf-8 -*-
"""
/***************************************************************************
 BFVWtools
                                 A QGIS plugin
 Plugin voor analyseren en bewerken van de BFVW data.
                              -------------------
        begin                : 2017-09-07
        git sha              : $Format:%H$
        copyright            : (C) 2017 by N. Veenstra (BFVW)
        email                : N.Veenstra55@gmail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
"""
from __future__ import print_function
from builtins import str
from builtins import object

#Importeren lybraries
    
import time
import datetime
import os
import ftplib
import os.path
import socket
import shutil
import qgis.utils
import sip
sip.setapi('QString', 2)

from qgis.utils import (
    iface
)

from qgis.core import (
    QgsApplication,
    QgsVectorLayer,
    QgsVectorFileWriter,
    QgsProject,
    Qgis
)

from qgis.gui import (
    QgsMessageBar
)

from PyQt5.QtCore import (
    QObject,
    pyqtSignal,
    Qt,
    QTimer,
    QSettings,
    QTranslator,
    qVersion,
    QCoreApplication
)

from PyQt5.QtWidgets import (
    QMessageBox,
    QMainWindow,
    QAction,
    QMenu,
    QFileDialog,
    QProgressBar
)
from PyQt5.QtGui import (
    QIcon
)

""" Initialize Qt resources from file resources.py """
from .resources import *

""" Import the code for the dialog """
from .bfvw_tools_dialog import BFVWtoolsDialog
from .bfvw_tools_dialog1 import BFVWtoolsDialog1
from .bfvw_tools_dialog2 import BFVWtoolsDialog2
from .bfvw_tools_dialog3 import BFVWtoolsDialog3

class BFVWtools(object):
    """QGIS Plugin Implementation."""

    def __init__(self, iface):
        self.iface = iface
        
        """ initialize plugin directory """
        self.plugin_dir = os.path.dirname(__file__)
        
        """ initialize locale """
        locale = QSettings().value('locale/userLocale')[0:2]
        locale_path = os.path.join(self.plugin_dir, 'i18n', 'BFVWtools_{}.qm'.format(locale))

        if os.path.exists(locale_path):
            self.translator = QTranslator()
            self.translator.load(locale_path)

            if qVersion() > '4.3.3':
                QCoreApplication.installTranslator(self.translator)


        """ Declare instance attributes """
        self.actions = []
        self.menu = self.tr(u'&BFVW Tools')

        """ TODO: We are going to let the user set this up in a future iteration """
        self.toolbar = self.iface.addToolBar(u'BFVW tools')
        self.toolbar.setObjectName(u'BFVW tools')

        """Dialoogvensters configureren"""
        self.dlg = BFVWtoolsDialog()
        self.dlg1 = BFVWtoolsDialog1()
        self.dlg2 = BFVWtoolsDialog2()
        self.dlg3 = BFVWtoolsDialog3()

        """Button [Bladeren] klikbaar maken"""
        self.dlg1.pushButton_1.clicked.connect(self.select_output_file1)
        self.dlg2.pushButton_2.clicked.connect(self.select_output_file2)
        self.dlg2.pushButton_2a.clicked.connect(self.select_output_file2a)
        self.dlg3.pushButton_3.clicked.connect(self.select_output_file3)

    """ noinspection PyMethodMayBeStatic """
    def tr(self, message):
        """Get the translation for a string using Qt translation API.

        We implement this ourselves since we do not inherit QObject.

        :param message: String for translation.
        :type message: str, QString

        :returns: Translated version of message.
        :rtype: QString
        """
        # noinspection PyTypeChecker,PyArgumentList,PyCallByClass
        return QCoreApplication.translate('BFVW tools', message)
        
    def add_action(
        self,
        icon_path,
        text,
        callback,
        enabled_flag=True,
        add_to_menu=True,
        add_to_toolbar=True,
        status_tip=None,
        whats_this=None,
        parent=None):

        
        """Add a toolbar icon to the toolbar.

        :param icon_path: Path to the icon for this action. Can be a resource
            path (e.g. ':/plugins/foo/bar.png') or a normal file system path.
        :type icon_path: str

        :param text: Text that should be shown in menu items for this action.
        :type text: str

        :param callback: Function to be called when the action is triggered.
        :type callback: function

        :param enabled_flag: A flag indicating if the action should be enabled
            by default. Defaults to True.
        :type enabled_flag: bool

        :param add_to_menu: Flag indicating whether the action should also
            be added to the menu. Defaults to True.
        :type add_to_menu: bool

        :param add_to_toolbar: Flag indicating whether the action should also
            be added to the toolbar. Defaults to True.
        :type add_to_toolbar: bool

        :param status_tip: Optional text to show in a popup when mouse pointer
            hovers over the action.
        :type status_tip: str

        :param parent: Parent widget for the new action. Defaults None.
        :type parent: QWidget

        :param whats_this: Optional text to show in the status bar when the
            mouse pointer hovers over the action.

        :returns: The action that was created. Note that the action is also
            added to self.actions list.
        :rtype: QAction
        """

    """ Create the dialog (after translation) in plugin menu and keep reference """
        #self.dlg = BFVWtoolsDialog()
        
        #icon = QIcon(icon_path)
        #action = QAction(icon, text, parent)
        #action.triggered.connect(callback)
        #action.setEnabled(enabled_flag)#

        #if status_tip is not None:
        #    action.setStatusTip(status_tip)

        #if whats_this is not None:
        #    action.setWhatsThis(whats_this)

        #if add_to_toolbar:
        #    self.toolbar.addAction(action)

        #if add_to_menu:
        #    self.iface.addPluginToMenu(
        #        self.menu,
        #        action)

        #self.actions.append(action)

        #return action     


    def initGui(self):
        """Create menu entries and toolbar icons inside the QGIS GUI."""
        
        icon_path = ":/plugins/bfvw_tools/"

        #Create the menu entrie Configuratie and toolbar icon inside the QGIS GUI.
        self.add_action(icon_path + "/icon.png", text=self.tr(u'Configuratie'), 
        callback=self.run, parent=self.iface.mainWindow())
        self.action = QAction( QIcon( os.path.dirname(os.path.realpath(__file__)) + "/icon.png" ), "Configuratie", 
        self.iface.mainWindow() )
        self.action.triggered.connect(self.run)        
    
        #Create the menu entrie Update CSV data and toolbar icon inside the QGIS GUI.
        self.add_action(icon_path + "/icon1.png", text=self.tr(u'Update CSV data'), 
        callback=self.run1, parent=self.iface.mainWindow())
        self.action1 = QAction( QIcon( os.path.dirname(os.path.realpath(__file__)) + "/icon1.png" ), "Update CSV data", 
        self.iface.mainWindow() )
        self.action1.triggered.connect(self.run1)        

        #Create the menu entrie CSV naar Shape CSV data and toolbar icon inside the QGIS GUI.
        self.add_action(icon_path + "/icon2.png", text=self.tr(u'CSV naar Shape'), 
        callback=self.run2, parent=self.iface.mainWindow())
        self.action2 = QAction( QIcon( os.path.dirname(os.path.realpath(__file__)) + "/icon2.png" ), "CSV naar Shape", 
        self.iface.mainWindow() )
        self.action2.triggered.connect(self.run2)        

        #Create the menu entrie XXXXXXXXXX and toolbar icon inside the QGIS GUI.
        self.add_action(icon_path + "/icon3.png", text=self.tr(u'XXXXXXXX'), 
        callback=self.run3, parent=self.iface.mainWindow())
        self.action3 = QAction( QIcon( os.path.dirname(os.path.realpath(__file__)) + "/icon3.png" ), "XXXXXXXX", 
        self.iface.mainWindow() )
        self.action3.triggered.connect(self.run3)       


        """ Add a custom menu """
        self.menu = QMenu( "&BFVW tools", self.iface.mainWindow().menuBar() )
        self.menu.setObjectName( "&BFVW tools" )
        actions = self.iface.mainWindow().menuBar().actions()
        lastAction = actions[-1]
        
        self.iface.mainWindow().menuBar().insertMenu( lastAction, self.menu )
        self.menu.addAction( self.action )        
        self.iface.mainWindow().menuBar().insertMenu( lastAction, self.menu )
        self.menu.addAction( self.action1 )
        self.iface.mainWindow().menuBar().insertMenu( lastAction, self.menu )
        self.menu.addAction( self.action2 )
        self.iface.mainWindow().menuBar().insertMenu( lastAction, self.menu )
        self.menu.addAction( self.action3 )

        """ Remove a QGIS menu (e.g., the Edit menu) """
        editMenu = self.iface.editMenu()
        editMenu.menuAction().setVisible( False )


        """ Add a custom toolbar """
        self.toolbar = self.iface.addToolBar( "Werkbalk BFVW tools" )
        self.toolbar.setObjectName( "BFVW tools" )
        self.toolbar.addAction( self.action )
        self.toolbar.addAction( self.action1 )
        self.toolbar.addAction( self.action2 )
        self.toolbar.addAction( self.action3 )

    def unload(self):
        """Removes the plugin menu item and icon from QGIS GUI."""
        for action in self.actions:
            self.iface.removePluginMenu(
                self.tr(u'&BFVW Tools'),
                action)
            self.iface.removeToolBarIcon(action)
                
        """" remove the toolbar """
        del self.toolbar

    """Use getSaveFileName for set file path or getExistingDirectory for set directory path"""    
      
    def select_output_file1(self):
        lastdir1 = QFileDialog.getExistingDirectory(self.dlg1, "Selecteer output directory ")
        QgsProject.instance().writeEntry("bfvwtools", "last_dir1", (lastdir1))
        self.dlg1.lineEdit_1.setText(lastdir1)
               
    def select_output_file2(self):
        lastdir2 = QFileDialog.getExistingDirectory(self.dlg2, "Selecteer intput directory ")
        QgsProject.instance().writeEntry("bfvwtools", "last_dir2", (lastdir2))
        self.dlg2.lineEdit_2.setText(lastdir2)

    def select_output_file2a(self):
        lastdir2a = QFileDialog.getExistingDirectory(self.dlg2, "Selecteer output directory ")
        QgsProject.instance().writeEntry("bfvwtools", "last_dir2a", (lastdir2a))
        self.dlg2.lineEdit_2a.setText(lastdir2a)
       
    def select_output_file3(self):
        lastdir3 = QFileDialog.getExistingDirectory(self.dlg3, "Selecteer in-output xxx ")
        QgsProject.instance().writeEntry("bfvwtools", "last_dir3", (lastdir3))
        self.dlg3.lineEdit_3.setText(lastdir3)

    """def select_output_file4(self):
        filename4 = QFileDialog.getSaveFileName(self.dlg4, "Selecteer in-output file ","", '*.ext')
        QgsProject.instance().writeEntry("bfvwtools", "filename4", (filename4))
        self.dlg4.lineEdit_4.setText(filename4)"""

###------------------------------------ Configuratie -------------------------------------------------------###

    def run(self):        
        project = QgsProject.instance()
        if project.fileName() == "":
            iface.messageBar().pushMessage("Let op !!!", "Er is nog geen project geladen !!!", level=Qgis.Critical, duration=5)
     
        else:
            """show the dialog"""
            self.dlg.show()
            
            hostname = QgsProject.instance().readEntry("bfvwtools", 'hostname')[0]
            self.dlg.lineEdit_1.setText(hostname)           
            
            username = QgsProject.instance().readEntry("bfvwtools", 'username')[0]
            self.dlg.lineEdit_2.setText(username)           

            password = QgsProject.instance().readEntry("bfvwtools", 'password')[0]
            self.dlg.lineEdit_3.setText(password)
                                    
            """Run the dialog event loop"""            
            result = self.dlg.exec_()
            """See if OK was pressed"""            
            if result:
                #Write variables to project 
                hostname = self.dlg.lineEdit_1.text()
                QgsProject.instance().writeEntry("bfvwtools", "hostname", (hostname))
                self.dlg.lineEdit_1.setText(hostname)
            
                username = self.dlg.lineEdit_2.text()
                QgsProject.instance().writeEntry("bfvwtools", "username", (username))
                self.dlg.lineEdit_2.setText(username)
            
                password = self.dlg.lineEdit_3.text()
                QgsProject.instance().writeEntry("bfvwtools", "password", (password))
                self.dlg.lineEdit_3.setText(password)

                #Write project
                project.write()

                QMessageBox.information( self.iface.mainWindow(), "Configuratie", 
                    "Configuratie opgeslagen.", QMessageBox.Ok )

                #Reload project after updatefiles
                self.extent = self.iface.mapCanvas().extent()
                filename = QgsProject.instance().fileName()
                if filename == '':
                    self.iface.messageBar()
                else:
                    self.iface.messageBar().pushMessage("Moment !!!", "Project word opnieuw geladen !!!", level=Qgis.Info, duration=5)
                    self.iface.projectRead.connect(self.set_extent)
                    self.iface.addProject(filename)
                    
    def set_extent(self):
        self.iface.mapCanvas().setExtent(self.extent)
        self.iface.projectRead.disconnect(self.set_extent)

###---------------------------- Update CSV files from FTP, Processbar ---------------------------------------------###
    
    def run1(self):
        project = QgsProject.instance()
        host = QgsProject.instance().readEntry ('bfvwtools', 'hostname')[0]
        user = QgsProject.instance().readEntry ('bfvwtools', 'username')[0]
        passw = QgsProject.instance().readEntry ('bfvwtools', 'password')[0]

        if project.fileName() == "":
            iface.messageBar().pushMessage("Let op !!!", "Er is nog geen project geladen !!!" ,level=Qgis.Critical, duration=10)
    
        elif host == "" or user == "" or passw == "":
            iface.messageBar().pushMessage("Let op !!!", "U heeft nog niet alle configuratie gegevens ingevoerd !!!", level=Qgis.Warning, duration=10)

        else:
            """Get setting from project file and fill lineEdit textbar"""

            self.dlg1.lineEdit_1.clear()
            last_dir1 = QgsProject.instance().readEntry("bfvwtools", 'last_dir1', "Blader naar uw CSV directory")[0]
            self.dlg1.lineEdit_1.setText(str(last_dir1))

            """Show the dialog"""            
            self.dlg1.show()
            """Run the dialog event loop"""
            result = self.dlg1.exec_()
            """See if OK was pressed"""
            if result:
                #Show processbar
                widget = self.iface.messageBar().createMessage("Een moment a.u.b."," Downloading CSV files . . .")
                prgBar = QProgressBar()
                prgBar.setAlignment(Qt.AlignLeft)
                prgBar.setValue(0)
                prgBar.setMaximum(100)           
                widget.layout().addWidget(prgBar)
                self.iface.messageBar().pushWidget(widget, Qgis.Warning)
                
                #Locale variables
                date = datetime.datetime.now()
                EXT = ".csv"
                FILE1 = "hoofdsoorten"
                FILE2 = "verliezen"
                FILE3 = "overigesoorten"
                FILE4 = "alarm1"
                FILE5 = "alarm2"
                FILE6 = "alarm3"
                FILE7 = "broedzorg"
                FILE8 = "nestkasten"

                #File and data variables
                CSVDIR = str(self.dlg1.lineEdit_1.text())+ '/'+ str(date.year)+'/'
            
                if not os.path.exists(CSVDIR):
                    os.makedirs(CSVDIR)
                
                prgBar.setValue(10)
                
                CONV1 = CSVDIR+FILE1+EXT
                CONV2 = CSVDIR+FILE2+EXT
                CONV3 = CSVDIR+FILE3+EXT
                CONV4 = CSVDIR+FILE4+EXT
                CONV5 = CSVDIR+FILE5+EXT
                CONV6 = CSVDIR+FILE6+EXT
                CONV7 = CSVDIR+FILE7+EXT
                CONV8 = CSVDIR+FILE8+EXT

                #FTP inlog
                try:
                  f = ftplib.FTP(host,user,passw)
                except (socket.error, socket.gaierror) as e:
                  print('ERROR: Kan geen verbinding met "%s" maken' % HOST)
                print('*** Verbinden met FTP server ***')
                print ("")
                
                prgBar.setValue(20)
                
                #Download files
                try:
                  f.retrbinary('RETR %s' % FILE1+EXT,
                     open(CSVDIR+FILE1+EXT, 'wb').write)
                except ftplib.error_perm:
                  print('ERROR: Kan file "%s" niet ophalen' % FILE1+EXT)
                  os.unlink(FILE1+EXT)
                else:
                  print('*** Download "%s" naar harddisk' % FILE1)
                
                prgBar.setValue(30)
                
                try:
                  f.retrbinary('RETR %s' % FILE2+EXT,
                     open(CSVDIR+FILE2+EXT, 'wb').write)
                except ftplib.error_perm:
                  print('ERROR: Kan file "%s" niet ophalen' % FILE2+EXT)
                  os.unlink(FILE2+EXT)
                else:
                  print('*** Download "%s" naar harddisk' % FILE2)

                prgBar.setValue(40)
                
                try:
                  f.retrbinary('RETR %s' % FILE3+EXT,
                     open(CSVDIR+FILE3+EXT, 'wb').write)
                except ftplib.error_perm:
                  print('ERROR: Kan file "%s" niet ophalen' % FILE3+EXT)
                  os.unlink(FILE3+EXT)
                else:
                  print('*** Download "%s" naar harddisk' % FILE3)

                prgBar.setValue(50)

                try:
                  f.retrbinary('RETR %s' % FILE4+EXT,
                     open(CSVDIR+FILE4+EXT, 'wb').write)
                except ftplib.error_perm:
                  print('ERROR: Kan file "%s" niet ophalen' % FILE4+EXT)
                  os.unlink(FILE4+EXT)
                else:
                  print('*** Download "%s" naar harddisk' % FILE4)

                prgBar.setValue(60)

                try:
                  f.retrbinary('RETR %s' % FILE5+EXT,
                     open(CSVDIR+FILE5+EXT, 'wb').write)
                except ftplib.error_perm:
                  print('ERROR: Kan file "%s" niet ophalen' % FILE5+EXT)
                  os.unlink(FILE5+EXT)
                else:
                  print('*** Download "%s" naar harddisk' % FILE5)

                prgBar.setValue(70)

                try:
                  f.retrbinary('RETR %s' % FILE6+EXT,
                     open(CSVDIR+FILE6+EXT, 'wb').write)
                except ftplib.error_perm:
                  print('ERROR: Kan file "%s" niet ophalen' % FILE6+EXT)
                  os.unlink(FILE6+EXT)
                else:
                  print('*** Download "%s" naar harddisk' % FILE6)

                prgBar.setValue(80)

                try:
                  f.retrbinary('RETR %s' % FILE7+EXT,
                     open(CSVDIR+FILE7+EXT, 'wb').write)
                except ftplib.error_perm:
                  print('ERROR: Kan file "%s" niet ophalen' % FILE7+EXT)
                  os.unlink(FILE7+EXT)
                else:
                  print('*** Download "%s" naar harddisk' % FILE7)

                prgBar.setValue(90)

                try:
                  f.retrbinary('RETR %s' % FILE8+EXT,
                     open(CSVDIR+FILE8+EXT, 'wb').write)
                except ftplib.error_perm:
                  print('ERROR: Kan file "%s" niet ophalen' % FILE8+EXT)
                  os.unlink(FILE8+EXT)
                else:
                  print('*** Download "%s" naar harddisk' % FILE8)

                f.quit()

                prgBar.setValue(100)

                self.iface.messageBar().clearWidgets()
                self.iface.mapCanvas().refresh()

                project.write()

                #Comfirm OK
                QMessageBox.information( self.iface.mainWindow(), "Update CSV data",
                    "Uw CSV data is bijgewerkt!", QMessageBox.Ok )
                
                #Reload project after updatefiles
                self.extent = self.iface.mapCanvas().extent()
                filename = QgsProject.instance().fileName()
                if filename == '':
                    self.iface.messageBar()
                else:
                    self.iface.messageBar().pushMessage("Moment !!!", "Project word opnieuw geladen!!", Qgis.Info, duration=5)
                    self.iface.projectRead.connect(self.set_extent)
                    self.iface.addProject(filename)

    def set_extent(self):
        self.iface.mapCanvas().setExtent(self.extent)
        self.iface.projectRead.disconnect(self.set_extent)

###-------------CSV files omzetten naar Shapefiles, Processbar --------------------###
    def run2(self):
        """Get setting from project file and fill lineEdit textbar"""
        self.dlg2.lineEdit_2.clear()
        last_dir2 = QgsProject.instance().readEntry("bfvwtools", 'last_dir2', "Blader naar uw CSV directory")[0]
        self.dlg2.lineEdit_2.setText(last_dir2)

        self.dlg2.lineEdit_2a.clear()
        last_dir2a = QgsProject.instance().readEntry("bfvwtools", 'last_dir2a', "Blader naar uw ShapeFile directory")[0]
        self.dlg2.lineEdit_2a.setText(last_dir2a)

        """Show the dialog"""
        self.dlg2.show()
        """Run the dialog event loop"""
        result = self.dlg2.exec_()
        """See if OK was pressed"""
        if result:
            #Show processbar
            widget = self.iface.messageBar().createMessage("Een moment a.u.b."," CSV files worden omgezet naar Shapefiles . . .")
            prgBar = QProgressBar()
            prgBar.setAlignment(Qt.AlignLeft)
            prgBar.setValue(0)
            prgBar.setMaximum(100)           
            widget.layout().addWidget(prgBar)
            self.iface.messageBar().pushWidget(widget, Qgis.Warning)

            """for i in range(1,100):
                   time.sleep(0.1) 
                   prgBar.setValue(i + 1)"""
                
            prgBar.setValue(10)
            time.sleep(0.1)
            
            #Convert CSV to Shapefiles
            CSVDIR = str(self.dlg2.lineEdit_2.text())+'/'            
            DESTDIR = str(self.dlg2.lineEdit_2a.text())+'/'
            
            prgBar.setValue(20)
            time.sleep(0.1)
                       
            if not os.path.exists(DESTDIR):
                os.makedirs(DESTDIR)
                
            prgBar.setValue(30)
            time.sleep(0.1)
            t = 40
            
            for root, dirs, files in os.walk(CSVDIR):
                for file in files:
                    if file.endswith('.csv'):
                        fullname = os.path.join(root, file).replace('/', '/')
                        filename = os.path.splitext(os.path.basename(fullname))[0]
                        
                        prgBar.setValue(t)
                        time.sleep(0.1)
                        t = t + 10
                        
                        uri = 'file:///%s?crs=%s&delimiter=%s&xField=%s&yField=%s&decimal=%s&useHeader=no&' % (fullname, 'EPSG:4326', ';', 'Field_7', 'Field_6', ',')
                        layer = QgsVectorLayer(uri, 'my_layer', 'delimitedtext')
                        QgsVectorFileWriter.writeAsVectorFormat(layer, DESTDIR + '/' + filename + '.shp', 'CP1250', layer.crs(), 'ESRI Shapefile')
                        
            prgBar.setValue(100)
            time.sleep(0.1)         
         
            self.iface.messageBar().clearWidgets()
            self.iface.mapCanvas().refresh()

            QMessageBox.information( self.iface.mainWindow(), "CSV naar Shape",
                "Uw Shapefiles zijn aangemaakt!", QMessageBox.Ok )

###-------------------- xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx --------------------###

    def run3(self):
        """Read data from projectfile"""
        project = QgsProject.instance()
        host = QgsProject.instance().readEntry ('bfvwtools', 'hostname')[0]
        user = QgsProject.instance().readEntry ('bfvwtools', 'username')[0]
        passw = QgsProject.instance().readEntry ('bfvwtools', 'password')[0]

        if project.fileName() == "":
            iface.messageBar().pushMessage("Let op !!!", "Er is nog geen project geladen !!!", level=Qgis.Critical, duration=10)

        elif host == "" or user == "" or passw == "":
            iface.messageBar().pushMessage("Let op !!!", "U heeft nog niet alle configuratie gegevens ingevoerd !!!", level=Qgis.Warning, duration=10)

        else:
            """Get setting from project file and fill lineEdit textbar"""
            self.dlg3.lineEdit_3.clear()
            last_dir3 = QgsProject.instance().readEntry("bfvwtools", 'last_dir3', "Blader naar uw CSV directory")[0]
            self.dlg3.lineEdit_3.setText(str(last_dir3))
            """Show the dialog"""            
            self.dlg3.show()
            """Run the dialog event loop"""
            result = self.dlg3.exec_()
            """See if OK was pressed"""
            if result:
                """Show processbar"""
                widget = self.iface.messageBar().createMessage("Een moment a.u.b."," We zijn druk bezig. . .")
                prgBar = QProgressBar()
                prgBar.setAlignment(Qt.AlignLeft)
                prgBar.setValue(0)
                prgBar.setMaximum(100)           
                widget.layout().addWidget(prgBar)
                self.iface.messageBar().pushWidget(widget, Qgis.Warning, duration=5)

                """for i in range(1,50):
                    time.sleep(0.1) 
                    prgBar.setValue(i + 1)"""

                prgBar.setValue(10)
                time.sleep(0.1)
                prgBar.setValue(20)
                time.sleep(0.1)
                prgBar.setValue(30)
                time.sleep(0.1)
                prgBar.setValue(40)
                time.sleep(0.1)
                prgBar.setValue(50)
                time.sleep(0.1)
                prgBar.setValue(60)
                time.sleep(0.1)
                prgBar.setValue(70)
                time.sleep(0.1)
                prgBar.setValue(80)
                time.sleep(0.1)
                prgBar.setValue(90)
                time.sleep(0.1)
                prgBar.setValue(100)
                time.sleep(0.1)
                
                self.iface.messageBar().clearWidgets()
                self.iface.mapCanvas().refresh()
           
            QMessageBox.information( self.iface.mainWindow(), "XXXXXXXX", 
                "XXXXXXXX is klaar!", QMessageBox.Ok )

###-------------------- xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx --------------------###
